/* Copyright (c) 1997, MpegTV LLC */
#ifndef _config_h
#define _config_h
#ifndef lint
static const char rcsid_config_h[] = "$Header: /net/hopkins/home/tristan/mpeg/include/RCS/config.h,v 1.47 1999/09/08 12:53:21 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/***************************************************************************/
/* By default ignore following storage qualifiers */
#define near
#define register

/***************************************************************************/
/*
 * Notes on basic data types:
 *
 * Integer representation is assumed to be 2's complement.
 * Portability is therefore a problem for achitectures or compilers that do not
 * use 2's complement representation.
 *
 * "char" must hold EXACTLY 8 bits.  This is always the case, I think.
 *
 * When the sign of "char" matters, the types "s_char" (for signed char) and
 * "u_char" (for unsigned char) should be used in the code.
 *
 * Unfortunately, most of the code currently assumes that "char" is SIGNED.
 * Portability is therefore a problem for achitectures or compilers where
 * "char" is unsigned.
 *
 * Note that "s_char" is guaranteed to be signed only if ANSI-C is used.
 * if a non-ANSI compiler is used, portability is not possible is
 * "char" is unsigned.
 *
 * "short" should hold EXACTLY 16 bits.  This is always the case, I think.
 * If not, the code may be broken and should be check carefully.
 *
 * "long" must hold at least 32 bits.  This is always the case, I think.
 *
 * In most of the code, it is assumed that "int" must hold at least 32 bits.
 * Portability is therefore a problem for achitectures or compilers
 * where "int" holds only 16 bits (like 8086/IBM-PCs).
 */

/*
 * INT16	is the signed type that holds EXACTLY 16 bits
 * INT32	is the signed type that holds EXACTLY 32 bits
 * INT64	is the signed type that holds EXACTLY 64 bits (if any)
 *
 * Those macros are used for defining the types int16, int32, u_int16 and u_int32,
 * int64 and u_int64. Those types maybe used for some particular code optimization.
 *
 * They should be redefined if values do not match defaults values:
 */
#define HAVE_INT16
#define INT16		short

#define HAVE_INT32
#define INT32		int

#ifdef __GNUC__
#	define HAVE_INT64
#	define INT64		long long
#endif

/* the following defaults may be undefined in the config files */
#define HAVE_STDIO_H
#define HAVE_TIME_H

/***************************************************************************/
#if defined(CONFIG_bsdi)
/* BSDi for x86 with gcc Compiler */

#include "config_bsdi.h"

#endif /* defined(CONFIG_bsdi) */

/***************************************************************************/
#if defined(CONFIG_sco)
/* SCO_SV 3.2 with gcc Compiler */

#include "config_sco.h"

#endif /* defined(CONFIG_sco) */

/***************************************************************************/
#if defined(CONFIG_solx86)
/* SunOS 5.x for x86 with gcc Compiler */

#include "config_solx86.h"

#endif /* defined(CONFIG_solx86) */

/***************************************************************************/
#if defined(CONFIG_sun5gcc) || defined(CONFIG_sun5) || defined(CONFIG_sol26)
/* SunOS 5.x with gcc or cc Compiler */

#include "config_sun5.h"

#endif /* defined(CONFIG_sun5gcc) || defined(CONFIG_sun5) || defined(CONFIG_sol26) */

/***************************************************************************/
#if defined(CONFIG_linux) || defined(CONFIG_linuxglibc) || defined(CONFIG_linuxglibc21) || defined(CONFIG_linuxglibcso)
/* PC 386 running linux with gcc Compiler */
/* CONFIG_linuxglibc: x86 running linux with gcc Compiler and glibc (libc6) [redhat 5.0] */

#include "config_linux.h"

#endif /* defined(CONFIG_linux) || defined(CONFIG_linuxglibc) */

/***************************************************************************/
#ifdef CONFIG_linuxalpha
/* alpha running linux with gcc Compiler */

#include "config_linuxalpha.h"

#endif /* CONFIG_linuxalpha */

/***************************************************************************/
#ifdef CONFIG_linuxppc
/* powermac running linux with gcc Compiler */

#include "config_linuxppc.h"

#endif /* CONFIG_linuxppc */

/***************************************************************************/
#if defined(CONFIG_sgigcc) || defined(CONFIG_sgi)
/* SGI with gcc or cc Compiler */

#include "config_sgi.h"

#endif /* CONFIG_sgi */

/***************************************************************************/
#if defined(CONFIG_hpgcc) || defined(CONFIG_hpcc)
/* HP with gcc or cc Compiler */

#include "config_hp.h"

#endif /* CONFIG_hpcc */

/***************************************************************************/
#if defined(CONFIG_du)
/* DIGITAL-UNIX with cc Compiler */

#include "config_du.h"

#endif /* CONFIG_du */

/***************************************************************************/
#if defined(CONFIG_win32)

#include "config_win32.h"

#endif /* CONFIG_win32 */

/***************************************************************************/
#if defined(CONFIG_wincemips)

#include "config_wincemips.h"

#endif /* CONFIG_wincemips */


/***************************************************************************/
#if defined(CONFIG_wincesh3)

#include "config_wincesh3.h"

#endif /* CONFIG_wincesh3 */

/***************************************************************************/
#if defined(CONFIG_wincesh4)

#include "config_wincesh4.h"

#endif /* CONFIG_wincesh4 */

/***************************************************************************/
#if defined(CONFIG_wincearm)

#include "config_wincearm.h"

#endif /* CONFIG_wincearm */

/***************************************************************************/
#if defined(CONFIG_wincex86)

#include "config_wincex86.h"

#endif /* CONFIG_wincex86 */

/***************************************************************************/
#if defined(CONFIG_wincex86em)

#include "config_wincex86em.h"

#endif /* CONFIG_wincex86em */

/***************************************************************************/
/* check that one configuration has been selected */
#ifndef CONFIG
	/* dirty trick: the directive #error is refused by the old cc ! */
	"CONFIG_$MPEG_CONFIG not recognized: fatal"
#error $MPEG_CONFIG not known by config.h: fatal
#endif

/***************************************************************************/

//#ifdef POCKET_PC
//#define USE_VOH_GAPI
//#endif

/*
 * Define NO_POSIX_SOURCE if MPEG_POSIX_SOURCE is not defined.
 * This dirty trick is used by some programs.
 */
#if !defined(NO_POSIX_SOURCE) && !defined(MPEG_POSIX_SOURCE)
#	define NO_POSIX_SOURCE
#endif /* !defined(NO_POSIX_SOURCE) && !defined(MPEG_POSIX_SOURCE) */

#ifndef MPEG_POSIX_SOURCE
#	undef _POSIX_SOURCE
#endif /* MPEG_POSIX_SOURCE */

/*
 * If NO_POSIX_SOURCE is defined, we want to force _POSIX_SOURCE macro
 * to be undefined, even if it was defined by the configuration segment.
 * The programs that do not comply with the POSIX.1 interface must
 * define NO_POSIX_SOURCE before including "config.h".
 */
#if 0
//#if defined(NO_POSIX_SOURCE) && defined(_POSIX_SOURCE)
//#	undef _POSIX_SOURCE
//#	undef MPEG_POSIX_SOURCE
//#endif /* defined(NO_POSIX_SOURCE) && defined(_POSIX_SOURCE) */
#endif /* 0 */

#ifndef CONFIG_next
#ifdef NO_POSIX_SOURCE
#	undef _POSIX_SOURCE
#	undef MPEG_POSIX_SOURCE
#endif /* NO_POSIX_SOURCE */
#endif /* not CONFIG_next */

/***************************************************************************/
#if 0 /* Tristan Savatier - Wed Jun 14 23:23:20 PDT 2000 */
//#ifdef __STDC__
//#	undef const
//#	undef volatile
//	/* define __STDCPP__ if the cpp is ANSI rather than traditional (K&R) */
//#define __STDCPP__ 
//#endif
//#else /* not __STDC__ */
//	/* If not ANSI C, ignore following ANSI storage qualifiers */
//#	define const
//#	define volatile
//#endif /* not __STDC__ */
#endif /* 0 - Tristan Savatier - Wed Jun 14 23:23:20 PDT 2000 */

#if defined(__STDC__) || defined(__cplusplus)
/* define __STDCPP__ if the cpp is ANSI rather than traditional (K&R) */
#ifndef __STDCPP__ 
#	define __STDCPP__ 
#endif
#endif /* not __STDC__ */

/***************************************************************************/

/* force no inline to debug */
#if defined(DEBUG) && !defined(NO_INLINE)
#	define NO_INLINE
#endif


/***************************************************************************/
/* this should be moved to individual config files */
#if defined(LINUX) || (defined(SOLARIS) && !defined(__i386__))
#define VCD		/* Video-CD support is built-in */
#endif

/***************************************************************************/

/* use INLINE for inline code */
#ifndef NO_INLINE
#	ifndef INLINE
#		define INLINE inline
#	endif
#else
#	undef INLINE
#	define INLINE
#endif


#endif /* _config_h */


