/* Copyright (c) 1998, MpegTV LLC */
#ifndef _sih_api_h
#define _sih_api_h
#ifndef lint
static const char rcsid_sih_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/sih_api.h,v 1.32 1999/08/03 11:12:08 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"
#include "msg.h"

#ifdef WIN32
#include "sih_file_win32_api.h"
#endif

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* current version */
#define MTVP_SIH_VERSION	4

/*
 * SIH_read may return the error status MTVP_SIH_READ_ERROR_TOO_SMALL to
 * indicate that the count passed is too small. This feature is very useful
 * for example when reading UDP sockets.  When reading a UDP socket, the
 * read count passed to the 'read' routine should always be at least
 * the maximum UDP packet size expected, otherwise some data can be lost.
 * MTVP_SIH_READ_ERROR_TOO_SMALL can therefore be returned if SIH_read is
 * called with a 'count' value smaller than the maximum UDP packet size expected.
 * Note that the packet payload size should be larger than this maximum
 * UDP packet size expected, otherwise SIH_read would always be called with
 * a count too small!
 */
#define MTVP_SIH_READ_ERROR_TOO_SMALL	(-2)
#define MTVP_SIH_READ_ERROR_TRY_AGAIN	(-3)

#if 0 /* Tristan Savatier - Sun Aug 23 23:48:12 PDT 1998 */
///* sih_callback() is now a macro */
//#define sih_feedback(data, size) (*SIH_callbacks.sih_feedback_ptr)(data, size)
#endif /* 0 - Tristan Savatier - Sun Aug 23 23:48:13 PDT 1998 */

/*
 * Handler id of built-in handlers.
 */
#define MTVP_SIH_HANDLER_ID_FILE			255
#ifndef WIN32
#define MTVP_SIH_HANDLER_ID_FD			254
#define MTVP_SIH_HANDLER_ID_VCD			253
#define MTVP_SIH_HANDLER_ID_PORT			252
#endif
#define MTVP_SIH_HANDLER_ID_HTTP_FTP		251

/* for the whence parameter of SIH_lseek */
#ifdef HAVE_STDIO_H
#define MTVP_SEEK_SET	SEEK_SET
#define MTVP_SEEK_CUR	SEEK_CUR
#define MTVP_SEEK_END	SEEK_END
#else
#define MTVP_SEEK_SET	0
#define MTVP_SEEK_CUR	1
#define MTVP_SEEK_END	2
#endif

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
/* handler data (opaque structure defined by each SIH handler) */
struct mtvp_sih_handler_data;

/* stream data (opaque structure defined by each SIH handler) */
struct mtvp_sih_stream_data;

/*
 * MTVP_SIB_IPL_SIZES is used to specify configuration of input buffer pool.
 * The buffer pool is a set of N small buffers of size M bytes.
 * Each small buffer contains a payload of P consecutive bytes.
 * Most of the times, P = M, but it is possible to indicate that
 * the payload is smaller that M, in which case the offset and
 * payload size must be specified. The offset D is the number of unused
 * bytes at the beginning of the small buffer, before the payload starts.
 * When P = M, D = 0. In all cases, D + P <= M.
 */ 
typedef struct {
	u_int	packet_count;			/* number of small buffers (N) */
	u_int	packet_gross_size;		/* size of each small buffer (M) */
	u_int	packet_payload_size;	/* small buffer payload size (P) */
	u_int	packet_offset;			/* offset to payload (D) */
	u_int	prefetch_trigger_list;
	u_int	prefetch_trigger_free;
} MTVP_SIB_IPL_SIZES;

/* callback routines etc */
typedef struct mtvp_sih_resources MTVP_SIH_RESOURCES;

struct mtvp_sih_resources {
	MTVP_MSG_STRUCT *msg;
	const void *z;
	int handler_id;
	int (*feedback)(const MTVP_SIH_RESOURCES *sih_resources, const void *data, size_t size);
};

/* block of data sent by application with COM_SIH_INIT */
typedef struct {
	int handler_id;
	char data[1];	/* 'p' passed to SIH_init is a ptr to data. this is a dummy declaration. in fact data may be any size */
} MTVP_SIH_INIT;

/* block of data sent by application with COM_SIH_CNTL */
typedef struct {
	int handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size */
} MTVP_SIH_CNTL;

/* Stream Input Interface (SIH), used by our SIB_IPL layer */
typedef struct {
	int	SIH_version;	/* handler must set this to MTVP_SIH_VERSION */
	int	(*SIH_probe_url)(const MTVP_SIH_RESOURCES *resources, const char *url);
	int (*SIH_init)(const MTVP_SIH_RESOURCES *resources, const char *p, struct mtvp_sih_handler_data **result_handler_data_ptr);
	void (*SIH_free)(struct mtvp_sih_handler_data *handler_data);
	int (*SIH_open_url)(struct mtvp_sih_handler_data *handler_data, const char *url, struct mtvp_sih_stream_data **result_stream_data_ptr);
	void (*SIH_close)(struct mtvp_sih_stream_data *stream_data);
	int (*SIH_fd)(struct mtvp_sih_stream_data *stream_data, void **result_fd_ptr);
	int (*SIH_read)(struct mtvp_sih_stream_data *stream_data, char *buf, size_t count, size_t *result_count_read);
//	int (*SIH_seek)(struct mtvp_sih_stream_data *stream_data, off_t offset, int whence, off_t *result_position);
	int (*SIH_seek)(struct mtvp_sih_stream_data *stream_data, off_t offset);
	int (*SIH_cntl)(struct mtvp_sih_handler_data *handler_data, const MTVP_SIH_CNTL *data, u_int size);
	int (*SIH_size)(struct mtvp_sih_stream_data *stream_data, off_t *result_size_ptr);
	int (*SIH_buffer)(struct mtvp_sih_stream_data *stream_data, MTVP_SIB_IPL_SIZES *sizes);
	void (*SIH_notify_free)(struct mtvp_sih_stream_data *stream_data, u_int number_of_free_buffers);
	void (*SIH_notify_rate)(struct mtvp_sih_stream_data *stream_data, double playback_rate);
} MTVP_SIH_EXPOSED;


/* block of data send received by application with A2G_COM_SIH_FEEDBACK */
typedef struct {
	int handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size */
} MTVP_SIH_FEEDBACK;


/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#if 0

#ifdef __cplusplus
extern "C" {
#endif

#ifndef WIN32
extern int _init(void);
extern int _fini(void);
#endif

extern int	SIH_register(MTVP_SIH_EXPOSED *exposed, int version);

#ifdef __cplusplus
}
#endif

#endif // 0

#endif /* _sih_api_h */
