/* Copyright (c) 1998, MpegTV LLC */
#ifndef _sih_file_win32_api_h
#define _sih_file_win32_api_h
#ifndef lint
static const char rcsid_sih_file_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/sih_file_api.h,v 1.104 1999/05/01 08:13:18 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/* SIH = Stream Input Handler */

#include "mtvp/misc.h"

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

/* type of feedback message */
enum sih_file_feedback_type {
	MTVP_SIH_FILE_FEEDBACK_TYPE_PLACEHOLDER = 0
};	


/* Specific VOH_FEEDBACK data from the GAPI Video Output Handler */
typedef struct {

	/* type of specific message */
	enum sih_file_feedback_type type;	

	/* associated data */
	union {
		char	placeholder;	
	} data;
	
} MTVP_SIH_FILE_FEEDBACK_DATA;

/* Specific A2G_COM_VOH_X_FEEDBACK data from the GAPI Video Output Handler */
typedef struct {
	int handler_id;		/* VOH_X_HANDLER */
	MTVP_SIH_FILE_FEEDBACK_DATA data;
} MTVP_SIH_FILE_FEEDBACK;


/* type of control message */
enum sih_file_cntl_type {
	MTVP_SIH_FILE_CNTL_TYPE_INIT_SUPERBUFFER_OFF = 0,	// must be called before stream is opened
	MTVP_SIH_FILE_CNTL_TYPE_INIT_SUPERBUFFER_ON = 1,	// must be called before stream is opened
	MTVP_SIH_FILE_CNTL_TYPE_ALLOCATE_SUPERBUFFER = 2,	// note: the superbuffer is always freed when the stream is closed.
	MTVP_SIH_FILE_CNTL_TYPE_FREE_SUPERBUFFER = 3,
	MTVP_SIH_FILE_CNTL_TYPE_DISABLE_SUPERBUFFER = 4,
	MTVP_SIH_FILE_CNTL_TYPE_ENABLE_SUPERBUFFER = 5
};	

typedef struct {
	DWORD size;	// size actually allocated, in bytes
} MTVP_SIH_FILE_CNTL_TYPE_ALLOCATE_SUPERBUFFER_OUTPUT;

typedef struct {
	DWORD size_max;	// max size of super buffer in bytes (e.g. (16 << 20) for 16 MB)
	DWORD size_inc;	// size increment in bytes (or rather, decrement, since we will start to allocate size_max and work down)
	DWORD size_safety;	// will make sure that this much memory is still free after super buffer is allocated

	MTVP_SIH_FILE_CNTL_TYPE_ALLOCATE_SUPERBUFFER_OUTPUT *output;	// output
} MTVP_SIH_FILE_CNTL_TYPE_ALLOCATE_SUPERBUFFER_PARAMS;

typedef struct {
	DWORD sib_buffer_size;

} MTVP_SIH_FILE_CNTL_TYPE_INIT_SUPERBUFFER_ON_PARAMS;


/* Specific MTVP_SIH_FILE_CNTL */
typedef struct {

	/* MTVP_SIH_FILE_CNTL structure must ALWAYS start with int handler_id */
	int handler_id;		/* MTVP_SIH_FILE_HANDLER */

	/* type of message */
	enum sih_file_cntl_type type;	

	/* associated data */
	union {
		char				nothing_yet;
		MTVP_SIH_FILE_CNTL_TYPE_INIT_SUPERBUFFER_ON_PARAMS init_superbuffer_on_params;
		MTVP_SIH_FILE_CNTL_TYPE_ALLOCATE_SUPERBUFFER_PARAMS allocate_superbuffer_params;
	} data;
	
} MTVP_SIH_FILE_CNTL;


#endif /* _sih_file_win32_api_h */
