/* Copyright (c) 1998, MpegTV LLC */
#ifndef _voh_api_h
#define _voh_api_h
#ifndef lint
static const char rcsid_voh_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/voh_api.h,v 1.115 1999/08/03 11:12:13 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/* VOH = Video Output Handler */

#include "mtvp/misc.h"

#include "mtvp/dith_im.h"
#include "mtvp/dither_api.h"
#include "mtvp/msg.h"

#ifdef USE_VOH_X
#include "mtvp/voh_x_api.h"
#endif

#ifdef WIN32
#include "mtvp/voh_win32_api.h"
#endif

#ifdef USE_VOH_GAPI
#include "mtvp/voh_gapi_api.h"
#endif

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#define MTVP_VOH_VERSION	9

#define MTVP_VOH_HANDLER_ID_X		255
#define MTVP_VOH_HANDLER_ID_SDL		254
#define MTVP_VOH_HANDLER_ID_NULL	253
#define MTVP_VOH_HANDLER_ID_WIN32	252
#define MTVP_VOH_HANDLER_ID_GAPI	251

/* max depth to match in TrueColor visuals */
#define MTVP_VOH_FULL_COLOR_MAX_DEPTH_DEFAULT	32

#define MTVP_VOH_RED		0
#define MTVP_VOH_GREEN	1
#define MTVP_VOH_BLUE	2
#define MTVP_VOH_RGB		3	/* size of arrays indexed by RED, GREEN, BLUE */

/* voh_control_flags */
#define MTVP_VOH_CONTROL_PEL_ASPECT_RATIO			0x0001


/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

typedef struct {
	u_char	r, g, b;
} MTVP_VOH_CLUT_ENTRY;

typedef struct {
    u_long		mask;
    u_char		number_of_bits_set;
    u_char		free_bits_at_bottom;
} MTVP_VOH_RGB_MASK;

/* generic window id */
typedef void *MTVP_VOH_WINDOW;

typedef struct mtvp_voh_resources MTVP_VOH_RESOURCES;

struct mtvp_voh_resources {
	MTVP_MSG_STRUCT *msg;
	const void *z;
	int handler_id;
	int (*feedback)(const MTVP_VOH_RESOURCES *voh_resources, const void *data, size_t size);
	int (*dith_class_init)(const MTVP_VOH_RESOURCES *voh_resources);
	void (*dith_class_free)(const MTVP_VOH_RESOURCES *voh_resources);
	u_int (*dith_get_clut)(const MTVP_VOH_RESOURCES *voh_resources, u_int max_colors, MTVP_VOH_CLUT_ENTRY *clut);
	void (*disp_refresh)(const MTVP_VOH_RESOURCES *voh_resources);
//	void (*notify_fd)(const MTVP_VOH_RESOURCES *voh_resources, void *fd);
	void *(*get_platform_specific_ptr)(const MTVP_VOH_RESOURCES *voh_resources);
};

typedef struct {
	u_int horizontal;	/* mpeg Horizontal_size, from which Frame_width is derived */
	u_int vertical;		/* mpeg Vertical_size, from which Frame_height is derived */
	u_int frame_width;		/* reconstructed frame width in pels (always multiple of MB) */
	u_int frame_height;		/* reconstructed frame height in pels (always multiple of MB) */
} MTVP_VOH_SIZE;

typedef enum {
	MTVP_VOH_ROTATION_NO,
	MTVP_VOH_ROTATION_90R,
	MTVP_VOH_ROTATION_90L,
	MTVP_VOH_ROTATION_180,
} MTVP_VOH_ROTATION;

typedef struct {
	double  pel_aspect_ratio;	/* 0 means free i.e. based on window geometry */
	MTVP_VOH_ROTATION  rotation;
} MTVP_VOH_CONTROL;

/* struct mtvp_voh_data is private to each VOH handler */
struct mtvp_voh_data;

/* block of data received by application with MTVP_MSG_NOTIFY_VOH_FEEDBACK */
typedef struct {
	int handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size */
} MTVP_VOH_FEEDBACK;

/* block of data sent by application with MTVP_MSG_COMMAND_VOH_CNTL */
typedef struct {
	int32 handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size and should start immediately after handler_id (no alignment data is allowed here) */
} MTVP_VOH_CNTL;


typedef struct {
	int	VOH_version;	/* handler must set this to VOH_VERSION */

	int (*VOH_init)(const MTVP_VOH_RESOURCES *voh_resources, MTVP_VOH_DITHERED_IMAGE *voh_dithered_image, const void *ptr, u_int size, struct mtvp_voh_data **voh_data_returned);
	void (*VOH_free)(struct mtvp_voh_data *voh_data);

	int (*VOH_window_create)(struct mtvp_voh_data *voh_data, MTVP_DITHER_TYPE dither_type, const MTVP_VOH_SIZE *voh_size, MTVP_VOH_WINDOW existing_window, const MTVP_VOH_CONTROL *voh_control, u_long voh_control_flags, void **fd_ptr_returned);
	void (*VOH_window_destroy)(struct mtvp_voh_data *voh_data);

	int (*VOH_resolve_dither)(struct mtvp_voh_data *voh_data, MTVP_VOH_WINDOW	existing_window, Bool dither_ok[MTVP_DITHER_COUNT], MTVP_DITHER_TYPE *prefered_type_ptr);

	int (*VOH_read_and_handle_all_pending_events)(struct mtvp_voh_data *voh_data, Bool must_handle_at_least_one_event);

	int (*VOH_dithered_image_alloc)(struct mtvp_voh_data *voh_data, 
			u_int *bytes_per_pel,	// out
			int *x_pitch,			// out
			int *y_pitch,			// out
			u_int *array_width,		// out
			u_int *array_height);	// out
	void (*VOH_dithered_image_free)(struct mtvp_voh_data *voh_data);

	Bool (*VOH_dithered_image_lock)(struct mtvp_voh_data *voh_data,
		u_char **array,				/* ptr to dest pels */	// out
		u_int   *src_x,				/* in src pels */	// out
		u_int   *src_y,				/* in src pels */	// out
		u_int   *src_width,			/* in src pels */	// out
		u_int   *src_height,		/* in src pels */	// out
		u_int   *dst_x,				/* in dest pels */	// out
		u_int   *dst_y				/* in dest pels */	// out
	);
	void (*VOH_dithered_image_unlock)(struct mtvp_voh_data *voh_data);

	int (*VOH_set_clut)(struct mtvp_voh_data *voh_data);
	void (*VOH_free_clut)(struct mtvp_voh_data *voh_data);
	int (*VOH_realize_clut)(struct mtvp_voh_data *voh_data);

	const u_char *(*VOH_get_clut_pixel_values)(struct mtvp_voh_data *voh_data);
	void (*VOH_get_rgb_masks)(struct mtvp_voh_data *voh_data, MTVP_VOH_RGB_MASK *masks);

	int (*VOH_set_control)(struct mtvp_voh_data *voh_data, MTVP_VOH_CONTROL *voh_control, u_long voh_control_flags);
	int (*VOH_cntl)(struct mtvp_voh_data *voh_data, const MTVP_VOH_CNTL *ptr, u_int size);

} MTVP_VOH_EXPOSED;

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

extern int _init(void);
extern int _fini(void);
extern int	VOH_register(MTVP_VOH_EXPOSED *exposed, int version);

#ifdef __cplusplus
}
#endif

#endif /* _voh_api_h */
