// PocketTV
//

#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"

#include "pockettv.hpp"
#include "registry.h"

#define FULLSCREEN_WINDOW_CLASS_NAME	TEXT("PocketTV Fullscreen")


/*----------------------------------------------------------------------
| FullscreenInit
+---------------------------------------------------------------------*/
int
PocketTV::FullscreenInit()
{
#ifdef POCKETTV_USE_FULLSCREEN
	int		status;
//	void	*value;

	WNDCLASS	wc;

	wc.style			= CS_HREDRAW | CS_VREDRAW | CS_NOCLOSE;
	wc.lpfnWndProc		= (WNDPROC) fullscreen_window_proc;
	wc.cbClsExtra		= 0;
	wc.cbWndExtra		= 0;
	wc.hInstance		= m_Instance;
	wc.hIcon			= (HICON)NULL;
	wc.hCursor			= 0;
	wc.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName		= 0;
	wc.lpszClassName	= FULLSCREEN_WINDOW_CLASS_NAME;

	if (RegisterClass(&wc) == 0) {
		status = GetLastError();
		_msg_printf("FullscreenInit: RegisterClass error %d\n", status);
		return status;
	}

	// use WS_EX_NOANIMATION otherwise this window appears in the HPC taskbar when active!
#if UNDER_CE < 212
	m_FullscreenWindow = CreateWindowEx(WS_EX_NOANIMATION /*| WS_EX_TOPMOST*/, FULLSCREEN_WINDOW_CLASS_NAME, TEXT(""), WS_POPUP,
#else
	m_FullscreenWindow = CreateWindowEx(WS_EX_NOANIMATION /*| WS_EX_TOPMOST*/, FULLSCREEN_WINDOW_CLASS_NAME, TEXT(""), WS_POPUP,
#endif
		0, 0,
		GetSystemMetrics(SM_CXSCREEN),
		GetSystemMetrics(SM_CYSCREEN),
		(HWND)NULL, (HMENU)NULL, m_Instance, NULL);

	if (m_FullscreenWindow == NULL) {
		status = GetLastError();
		_msg_printf("FullscreenInit: CreateWindow error %d\n", status);
		return status;
	}
	return 0;

#else // not POCKETTV_USE_FULLSCREEN
	return -1;
#endif // not POCKETTV_USE_FULLSCREEN
}

#ifdef POCKETTV_USE_FULLSCREEN
/*--------------------------------------------------------------------------+
| GetFullscreenRect
+--------------------------------------------------------------------------*/
void
PocketTV::GetFullscreenRect(RECT *pRect1, RECT *pRect2, RECT *pRect3) 
{
//	pRect->top = 0;
//	pRect->left = 0;
	RECT r1;
	RECT r2;

//#ifdef USE_VOH_GAPI
//	if (m_VOHHandlerId == MTVP_VOH_HANDLER_ID_WIN32) {
//#endif

	r1.top = 0;
	r1.left = 0;
	r1.bottom = GetSystemMetrics(SM_CYSCREEN);
	r1.right = GetSystemMetrics(SM_CXSCREEN);
	
//	SystemParametersInfo(SPI_GETWORKAREA, 0, &r1, 0);
//#ifdef POCKET_PC
//		r1.top = 0;
//#endif

#ifdef USE_VOH_GAPI
	if (!m_GapiDisabled) {
		r2.top = 0;
		r2.left = 0;
		r2.bottom = m_GapiInitOutput.gxdp.cyHeight;
		r2.right = m_GapiInitOutput.gxdp.cxWidth;
	} else {
#endif
		r2 = r1;
#ifdef USE_VOH_GAPI
	}
#endif

	if (pRect3 != NULL) {
		*pRect3 = r1;
	}

	if (pRect1 != NULL) {
//		if (m_FullscreenRotation == MTVP_VOH_ROTATION_90L
//							|| m_FullscreenRotation == MTVP_VOH_ROTATION_90R) {

		if (m_FullscreenRotation != m_NormalRotation) {
			pRect1->top = r1.left;
			pRect1->left = r1.top;
			pRect1->bottom = r1.right;
			pRect1->right = r1.bottom;
		} else {
			*pRect1 = r1;
		}
	}
	if (pRect2 != NULL) {
		if (m_FullscreenRotation == MTVP_VOH_ROTATION_90L
							|| m_FullscreenRotation == MTVP_VOH_ROTATION_90R) {
			pRect2->top = r2.left;
			pRect2->left = r2.top;
			pRect2->bottom = r2.right;
			pRect2->right = r2.bottom;
		} else {
			*pRect2 = r2;
		}
	}
}


/*----------------------------------------------------------------------
| DefaultFullscreenRotation
+---------------------------------------------------------------------*/
void
PocketTV::DefaultFullscreenRotation(MTVP_VOH_ROTATION rotation)
{
	int wmId;
	SetRegistryValueDWORD(TEXT("Rotation"), rotation);

	m_DefaultFullscreenRotation = rotation;

	switch (rotation) {
	case MTVP_VOH_ROTATION_90L:
		wmId = IDM_OPTIONS_FULLSCREEN_ROTATION_90L;
		break;
	case MTVP_VOH_ROTATION_90R:
	default:
		wmId = IDM_OPTIONS_FULLSCREEN_ROTATION_90R;
		break;
	}

	CheckMenuRadioItem(m_Menu, IDM_OPTIONS_FULLSCREEN_ROTATION_FIRST,
							  IDM_OPTIONS_FULLSCREEN_ROTATION_LAST,   
							  wmId,
							  MF_BYCOMMAND);

//	m_FullscreenRotation = GetFullscreenRotation(NULL);
	GetRotation(&m_FullscreenRotation, &m_NormalRotation);
}

/*----------------------------------------------------------------------
| PlayFullscreen
+---------------------------------------------------------------------*/
void
PocketTV::PlayFullscreen(bool yes)
{
//	MtvpAutoPlay(yes);
	SetRegistryValueDWORD(TEXT("PlayFullscreen"), yes);
	CheckMenuItem(m_Menu, IDM_OPTIONS_PLAY_FULLSCREEN, (yes ? MF_CHECKED : MF_UNCHECKED));
	m_PlayFullscreen = yes;
}

/*----------------------------------------------------------------------
| GetRotation0
+---------------------------------------------------------------------*/
void
PocketTV::GetRotation0(MTVP_VOH_ROTATION *fullscreen_rotation, MTVP_VOH_ROTATION *normal_rotation)
{
#ifdef POCKET_PC
	DWORD value;

	if (GetRegistryValueFromKeyDWORD(
			HKEY_LOCAL_MACHINE, TEXT("SYSTEM\\GDI\\DRIVERS"), TEXT("DispMode"), &value) == ERROR_SUCCESS) {
#define PORTRAIT_NORMAL                 0
#define LANDSCAPE_NORMAL                2
#define LANDSCAPE_LEFT_NORMAL   3

#define PORTRAIT_SUBPIXEL               4
#define LANDSCAPE_SUBPIXEL              6
#define LANDSCAPE_LEFT_SUBPIXEL 7

#define PORTRAIT_HIRES                  12
#define LANDSCAPE_HIRES                 14
#define LANDSCAPE_LEFT_HIRES    15

#define PORTRAIT_VGA                    28
#define LANDSCAPE_VGA                   30
#define LANDSCAPE_LEFT_VGA              31

		const TCHAR *driver_name;
		if (GetRegistryValueFromKeyAny(HKEY_LOCAL_MACHINE,
				TEXT("SYSTEM\\GDI\\DRIVERS"), TEXT("Display"), REG_SZ, (BYTE **)&driver_name, NULL) == ERROR_SUCCESS) {
			if (_wcsicmp(driver_name, TEXT("jsddi.dll")) == 0) {

				if (value & 1) {
					*fullscreen_rotation = *normal_rotation = MTVP_VOH_ROTATION_90R;
					return;
				}
				if (value & 2) {
					*fullscreen_rotation = *normal_rotation = MTVP_VOH_ROTATION_90L;
					return;
				}
				// (value & 3) == 0 : PORTRAIT MODE
			}
			free((void *)driver_name);
		}
	}

// These are some video extensions exposed through ExtEscape.
// There are more that are used internally by the NVD touch
// and keyboard drivers.
#define VD_GETCONFIG              202
#define VD_GETPHYSICALDIMENSIONS  204
#define VD_TOGGLEZOOM             205
#define VD_CHECKFORPRESENCE       207

// These are some result code.
#define VD_FAILURE                0
#define VD_SUCCESS                1
#define VD_PRESENT                3

// Here's an example of detecting NVD.
// It's unlikely that another video driver would
// respond to a 207 with a 3 so it's pretty safe.

	if (ExtEscape(GetDC(NULL), 
                  VD_CHECKFORPRESENCE,
                  0,
                  NULL,
                  0,
                  NULL ) == VD_PRESENT ) {

		if (GetRegistryValueFromKeyDWORD(
			HKEY_LOCAL_MACHINE, TEXT("SOFTWARE\\Nyditot\\Virtual Display"), TEXT("Orientation"), &value) == ERROR_SUCCESS) {
			if (value == 90) {
				*fullscreen_rotation = *normal_rotation = MTVP_VOH_ROTATION_90L;
				return;
			}
			if (value == 270) {
				*fullscreen_rotation = *normal_rotation = MTVP_VOH_ROTATION_90R;
				return;
			}
			if (value == 180) {
				*normal_rotation = MTVP_VOH_ROTATION_180;
				*fullscreen_rotation = m_DefaultFullscreenRotation;
				return;
			}
		}
	}
#endif // POCKET_PC

	if (
#ifdef USE_VOH_GAPI
		(!m_GapiDisabled && m_GapiInitOutput.gxdp.cyHeight > m_GapiInitOutput.gxdp.cxWidth) ||
#endif
		(m_GapiDisabled && (DWORD)GetSystemMetrics(SM_CYSCREEN) > (DWORD)GetSystemMetrics(SM_CXSCREEN))
		) {
//		m_FullscreenRotation = MTVP_VOH_ROTATION_90L;
//		m_FullscreenRotation = MTVP_VOH_ROTATION_90R;
	} else {
		*fullscreen_rotation = *normal_rotation = MTVP_VOH_ROTATION_NO;
		return;
	}

	*normal_rotation = MTVP_VOH_ROTATION_NO;
	*fullscreen_rotation = m_DefaultFullscreenRotation;
}

/*----------------------------------------------------------------------
| GetRotation
+---------------------------------------------------------------------*/
void
PocketTV::GetRotation(MTVP_VOH_ROTATION *fullscreen_rotation, MTVP_VOH_ROTATION *normal_rotation)
{
	GetRotation0(fullscreen_rotation, normal_rotation);

	// for video with normal aspect ratio (width > height), that should be good
	if (m_VideoInfos.width == 0 || m_VideoInfos.height == 0 || m_VideoInfos.width >= m_VideoInfos.height) {
		return;
	}
	RECT r;

#ifdef USE_VOH_GAPI
	if (!m_GapiDisabled) {
		r.top = 0;
		r.left = 0;
		r.bottom = m_GapiInitOutput.gxdp.cyHeight;
		r.right = m_GapiInitOutput.gxdp.cxWidth;
	} else {
#endif
		r.top = 0;
		r.left = 0;
		r.bottom = GetSystemMetrics(SM_CYSCREEN);
		r.right = GetSystemMetrics(SM_CXSCREEN);
#ifdef USE_VOH_GAPI
	}
#endif

	// for video with abnormal aspect ratio (width < height), we try to force a better fullscreen rotation mode
	// that should show more of the image.
	// for example, if width == 320 and height == 240, fullscreen should show all the image on a 240x320 screen.

	if (r.right - r.left < r.bottom - r.top) { // portrait screen
		*fullscreen_rotation = MTVP_VOH_ROTATION_NO;
	} else {
		// landscape screen
		*fullscreen_rotation = m_DefaultFullscreenRotation;
	}
}


#if !defined(POCKET_PC)

/***********************************************************************************************

***********************************************************************************************/
static void
TaskbarHide(void)
{
	HWND hWnd;
	hWnd = FindWindow(TEXT("HHTaskBar"), 0);
	if (hWnd) ShowWindow(hWnd, SW_HIDE);
}

/***********************************************************************************************

***********************************************************************************************/
static void
TaskbarShow(void)
{
	HWND hWnd;
	hWnd = FindWindow(TEXT("HHTaskBar"), 0);
	if (hWnd) ShowWindow(hWnd, SW_SHOWNA);
}
#endif


/*----------------------------------------------------------------------
| Fullscreen
+---------------------------------------------------------------------*/
void
PocketTV::Fullscreen(bool yes)
{
	bool	was_fullscreen = m_Fullscreen;

	if (m_FullscreenDisabled) {
		return;
	}

	if (m_Fullscreen == yes) {
		return;
	}

	if (m_CancelModePending) {
		ResumeVideo();
		m_CancelModePending = false;
	}

	m_DisableResumeVideo = true;

	if (yes) {
		SuspendVideo();
		m_Fullscreen = true;
	}
	if (yes) {
		RECT r;

#if !defined(POCKET_PC)
		TaskbarHide();
#endif

		GetRotation(&m_FullscreenRotation, &m_NormalRotation);
		GetFullscreenRect(NULL, &r, NULL);
		if (!m_GapiDisabled
				// && ((!m_FastVideoDisabled && m_FastVideo) || m_FastVideoDisabled)
				&& !IsStretched(m_FullscreenStretch, &r)) {

			if (m_VOHHandlerId == MTVP_VOH_HANDLER_ID_WIN32) {

				Gapi0(true, true);
			}
//			GapiSetVideoTargetRect();	// now done in Rotate
			Rotate(m_FullscreenRotation);
		} else {
			if (m_VOHHandlerId != MTVP_VOH_HANDLER_ID_WIN32) {

				Gapi0(false, true);
			}
			MtvpSetDisplayWindow((int)m_FullscreenWindow);
			if (m_FullscreenRotation == m_NormalRotation) {
				// case of fullscreen with no GAPI under a Landscape driver!
				Rotate(MTVP_VOH_ROTATION_NO);
			} else {
				Rotate(m_FullscreenRotation);
			}

			FullscreenSetVideoTargetRect();
			SetVideoStretch0(m_FullscreenStretch);

		}

	} else {
		m_Fullscreen = false;
		SuspendVideo();
//		SetForegroundWindow(m_Window);
//		ShowWindow(m_FullscreenWindow, SW_HIDE);
//		SetActiveWindow(m_Window);
//		SetFocus(m_Window);
	}

//	GapiSetVideoTargetRect();
	
	if (yes) {
		ShowWindow(m_FullscreenWindow, SW_SHOW);
		SetForegroundWindow(m_FullscreenWindow);
#if 0
		if (m_VOHHandlerId != MTVP_VOH_HANDLER_ID_WIN32) {
			MTVP_VOH_GAPI_CNTL data;

			data.handler_id = MTVP_VOH_HANDLER_ID_GAPI;
			data.type = MTVP_VOH_GAPI_CNTL_TYPE_CLEAR;
			MtvpSendCommandCustom(MTVP_COMMAND_VOH_CNTL, (const void *)&data, sizeof data);
		}
#endif // 0
//		SHFullScreen(m_FullscreenWindow, SHFS_HIDETASKBAR|SHFS_HIDESIPBUTTON);
	} else {
//		ShowWindow(m_Window, SW_SHOW);
//		SetForegroundWindow(m_Window);
//		SetActiveWindow(m_Window);
//		SetFocus(m_Window);
//		SetNavBarText(m_Window, m_NavBarTitle);

//		SuspendVideo();
//		Rotate(MTVP_VOH_ROTATION_NO);
//		MtvpSetDisplayWindow((int)m_Window);	// now done in PickVideoMode
//		SetVideoTargetRect();	// now done in PickVideoMode
		PickVideoMode();

		// hiding the m_FullscreenWindow will cause a WM_PAINT even, so we should do that
		// only once the video has been properly rotated.
		ShowWindow(m_FullscreenWindow, SW_HIDE);

#if !defined(POCKET_PC)
		TaskbarShow();
#endif
	}
	m_DisableResumeVideo = false;
	ResumeVideo();
}

/*----------------------------------------------------------------------
| FullscreenKillFocus
+---------------------------------------------------------------------*/
void
PocketTV::FullscreenKillFocus()
{
	if (m_Fullscreen) {
		m_WasPlayingBeforeBecommingInactive = (m_State == MTVP_STATE_PLAYING);
		SetState(MTVP_STATE_PAUSED);
	}
#if !defined(POCKET_PC)
	TaskbarShow();
#endif
}

/*----------------------------------------------------------------------
| FullscreenSetFocus
+---------------------------------------------------------------------*/
void
PocketTV::FullscreenSetFocus()
{
#ifdef SHFS_HIDETASKBAR
	SHFullScreen(m_FullscreenWindow, SHFS_HIDETASKBAR|SHFS_HIDESIPBUTTON);
#else
//	fullscreen_dialog(m_FullscreenWindow);
#endif
	MtvpFocus();
#if !defined(POCKET_PC)
	TaskbarHide();
#endif
}

/*----------------------------------------------------------------------
| fullscreen_window_proc
+---------------------------------------------------------------------*/
static LRESULT CALLBACK 
fullscreen_window_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_PAINT:
			{
				HDC hdc;
				PAINTSTRUCT ps;
				hdc = BeginPaint(hWnd, &ps);
				EndPaint(hWnd, &ps);
				Player->MtvpRepaint();
			}
			break;
//		case WM_ACTIVATE:
//			if (LOWORD(wParam) == WA_INACTIVE) {
//				player->SetState(MTVP_STATE_PAUSED);
//				player->Gapi0(false, true);
//			}
//			break;
		case WM_KILLFOCUS:
			Player->FullscreenKillFocus();
			break;
		case WM_SETFOCUS:
			Player->FullscreenSetFocus();
			break;
		case WM_LBUTTONDOWN:
//			Player->m_FullscreenActive = false;
			Player->Fullscreen(false);
			break;
//		case WM_DESTROY:
//			break;
#ifndef USE_VOH_GAPI
		case WM_SETTINGCHANGE:
			if (wParam == SPI_SETWORKAREA) {
				Player->FullscreenSetVideoTargetRect();
			}
			break;
#endif
		case WM_WINDOWPOSCHANGED: {
			const WINDOWPOS *p = (const WINDOWPOS *)lParam;
			// the following code should take care of the following situation on H/PC devices:
			// the player is in fullscreen mode (not GAPI) and the PocketTV tab of the Taskbar is clicked.
			// If we don't do that, the fullscreen windows stays on top instead of disapearing.
			if (p->flags == (SWP_NOOWNERZORDER|SWP_NOSIZE|SWP_NOMOVE|SWP_NOACTIVATE)) {
				ShowWindow(Player->m_FullscreenWindow, SW_HIDE);
			}
			}
			break;
//		case WM_TIMER:
//			_msg_printf("WM_TIMER Fullscreen\n");
//			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}
#endif // USE_FULLSCREEN


// eof
